﻿using Example_Variable_Driver.Configuration;
using System;
using System.Windows.Forms;
using MLogics.VisXpert.Online.Editor;

namespace Example_Variable_Driver
{
    public partial class RuntimeDialog : Form
    {
        private MLogics.VisXpert.Online.ModuleConnection ModCon;
        private ModbusServer Server;

        public RuntimeDialog()
        {
            InitializeComponent();
            StartDriver();
        }

        private void StartDriver()
        {
            ConnectVisXpert();
            LoadConfiguration();
            StartVariableServer();
            RefreshStatistics();
        }

        /// <summary>
        /// Connect to VisXpert and set up default Commands for Exit requests and Pop to front
        /// </summary>
        private void ConnectVisXpert()
        {
            ModCon= new MLogics.VisXpert.Online.ModuleConnection();
            ModCon.Connect();
            ModCon.ExitRequest += (a, b) => { this.Invoke(new Action (() => { this.Close(); })); };
            ModCon.PopToFront += (a, b) => { this.Invoke(new Action(() => { this.Show(); })); };
        }

        /// <summary>
        /// Load the Configured variables from VisXpert and prepare
        /// the Server for runtime
        /// </summary>
        private void LoadConfiguration()
        {
            Server = new ModbusServer();

            var GrpAdapter = new GroupConfigProvider(ModCon);
            foreach (ModbusGroupConfig GrpConfig in GrpAdapter.GetAsList(ModCon.ModuleName, () => { return new ModbusGroupConfig(); }))
            {
                var Grp = new ModbusServerGrp(GrpConfig);
                Server.Groups.Add(Grp);

                var VarAdpater = new VariableConfigProvider(ModCon);
                foreach (ModbusVarConfig VarConfig in VarAdpater.GetAsList(GrpConfig.GroupName, () => { return new ModbusVarConfig(); }))
                {
                    var Var = new ModbusServerVar(VarConfig);
                    Grp.Variables.Add(Var);
                }
            }
        }

        private void StartVariableServer()
        {
            Server.Connect(ModCon);
            timerStatistics.Start();
        }

        private void ShutDownDriver()
        { 
            Server.Disconnect();
            ModCon.Disconnect();
            timerStatistics.Stop();
        }

        private void RefreshStatistics()
        {
            try
            {
                var stats = ModbusServerStatisticsProvider.GetStatistics(Server);
                RefreshStatisticsItem("Groups", "Active", stats.ActiveGroupCount);
                RefreshStatisticsItem("Groups", "Total", stats.TotalGroupCount);
                RefreshStatisticsItem("Groups", "Connected", stats.ConnectedGroupCount);

                RefreshStatisticsItem("Variables", "Active", stats.ActiveVarCount);
                RefreshStatisticsItem("Variables", "Total", stats.TotalVarCount);
                RefreshStatisticsItem("Variables", "Connected", stats.ConnectedVarCount);
                RefreshStatisticsItem("Variables", "Advised", stats.AdvisedVarCount);
                RefreshStatisticsItem("Variables", "Error", stats.ErrorVarCount);

                RefreshStatisticsItem("Timings", "Average Response time", stats.AverageVarResponseTime);
            }
            catch (Exception) { System.Diagnostics.Debugger.Break(); }
        }

        private void RefreshStatisticsItem(string Group, string Name, object Value)
        {
            //Get or Add group if it does not exist yet
            var grp = listViewStats.Groups[Group];
            if (grp == null)
            {
                grp = listViewStats.Groups.Add(Group, Group);
            }

            var lvi = listViewStats.Items[Group + Name];
            if (lvi == null)
            {
                lvi = listViewStats.Items.Add(Group + Name, Name, 0);
                lvi.Group = grp;            
            }

            ListViewItem.ListViewSubItem slvi;
            if (lvi.SubItems.Count <=1)
            {
                slvi = lvi.SubItems.Add("");
            }
            else
            {
                slvi = lvi.SubItems[1];
            }

            if (Value == null)  slvi.Text = "";
            else                slvi.Text  = Value.ToString();
        }

        //_______________________________________________________________________
        #region Event Handlers
        private void toolStripButtonLog_Click(object sender, EventArgs e)
        {
            var form = new LogViewDialog();
            form.ShowDialog(this);
        }

        private void toolStripButtonErrors_Click(object sender, EventArgs e)
        {
            var form = new ErrorVarDialog(Server);
            form.ShowDialog(this);
        }

        private void RuntimeDialog_VisibleChanged(object sender, EventArgs e)
        {
            timerStatistics.Enabled = this.Visible;
        }

        private void FormRuntime_FormClosed(object sender, FormClosedEventArgs e)
        {
            ShutDownDriver();
        }

        private void timerStatistics_Tick(object sender, EventArgs e)
        {
            RefreshStatistics();
        }
        #endregion
    }
}
