﻿using System;
using Microsoft.Win32;

namespace BatchXpert_GP_Batch
{
    /// <summary>
    /// If the Assembly name contains "Visxpert" Try to load it from the VisXpert directory
    /// </summary>
    internal class VisXpertDirectoryAssemblyResolver
    {
        public static System.Reflection.Assembly Resolve(object sender, ResolveEventArgs e)
        {
            if (!e.Name.ToLower().Contains("visxpert"))
                return null;
            var AsmName = new System.Reflection.AssemblyName(e.Name);
            return TryLoadAssemblyFromDirectory(AsmName, GetVisXpertPath()+@"bin\");
        }

        private static string GetVisXpertPath()
        {
            string DefaulUpdateFileFolder = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFilesX86)+@"\VisXpert\";
            string TmpString;
            RegistryKey RegKey;
            RegKey=Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\VisXpert");

            if (RegKey is object)
            {
                TmpString=RegKey.GetValue("InstallLocation", DefaulUpdateFileFolder).ToString().Trim('"');

                if (TmpString.EndsWith(@"\"))
                {
                    return TmpString;
                }
                else
                {
                    return TmpString+@"\";
                }
            }
            else
            {
                RegKey=Registry.LocalMachine;
                RegKey=RegKey.OpenSubKey(@"Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\VisXpert\");

                if (RegKey is object)
                {
                    TmpString=RegKey.GetValue("InstallLocation", DefaulUpdateFileFolder).ToString().Trim('"');

                    if (TmpString.EndsWith(@"\"))
                    {
                        return TmpString;
                    }
                    else
                    {
                        return TmpString+@"\";
                    }
                }
                else
                {
                    return DefaulUpdateFileFolder;
                }
            }
        }

        private static System.Reflection.Assembly TryLoadAssemblyFromDirectory(System.Reflection.AssemblyName AsmName, string BaseDirectory)
        {
            System.IO.FileInfo fName;

            if (AsmName.Name.ToLower().EndsWith(".resources"))
            {
                fName=new System.IO.FileInfo(System.IO.Path.Combine(BaseDirectory, AsmName.CultureInfo.Name, AsmName.Name+".dll"));
                if (fName.Exists)
                    return System.Reflection.Assembly.LoadFrom(fName.FullName);
                fName=new System.IO.FileInfo(System.IO.Path.Combine(BaseDirectory, AsmName.CultureInfo.Parent.Name, AsmName.Name+".dll"));
                if (fName.Exists)
                    return System.Reflection.Assembly.LoadFrom(fName.FullName);
            }

            fName=new System.IO.FileInfo(BaseDirectory+AsmName.Name+".dll");
            if (fName.Exists)
                return System.Reflection.Assembly.LoadFrom(fName.FullName);
            return null;
        }
    }
}